import codecs
import json

import networkx as nx
from networkx.readwrite import json_graph

class NetworkxJson:
  def __init__(self, graph):
    self.graph = graph

  def nx_export(self, export_type, output_file):
    data = getattr(json_graph, "%s_data" % export_type)(self.graph)

    target = codecs.open(output_file, "w", "utf-8")
    json.dump(data,target, sort_keys=True, indent=2, ensure_ascii=False)
    print "export node-link: %s" % output_file


from wekeypedia.wikipedia_page import WikipediaPage

import networkx as nx

from multiprocessing import Pool
from multiprocessing.dummy import Pool as ThreadPool

class WikipediaNetwork:
  def __init__(self, keywords):
    self.keywords = keywords
    self.graph = nx.Graph()
    self.mapping = []

  def get_page_links(self, title):
    w = WikipediaPage(title)

    if (w.page):
      page = w.page.title
    else:
      page = ""

    self.mapping.append({
      'query'   : title,
      'page'    : page,
      'problem' : w.problem
      })

    return w.links()

  def method(self, r):
    concept  = r[0]
    links = r[1]

    print "## %s" % concept

    self.graph.add_node(concept)

    accepted_links = set(links) & set(self.keywords)

    for link in accepted_links:
      print "-> %s" % link
      self.graph.add_edge(concept, link)

    print "\r"

  def build(self):
    pool = ThreadPool(8)

    for kw in self.keywords:
      pool.apply_async(lambda kw: (kw, self.get_page_links(kw)), (kw, ), callback=self.method)

    pool.close()
    pool.join()


# -*- coding: utf-8 -*-

import csv

class Mapping:
  def __init__(self, mapping):
    self.mapping = mapping

  def csv(self, output):

    with open(output, "wb") as csv_file:
      w = csv.writer(csv_file)

      w.writerow([ "textbook", "wikipedia", "problem" ])

      for correspondance in self.mapping:
        w.writerow([ correspondance['query'],
          correspondance['page'],
          correspondance['problem'] ])


# -*- coding: utf-8 -*-
import sys

import wikipedia
import urllib

import requests

from colorama import Fore

from datetime import date

def url2title(url):
  title = url.split("/")

  if(len(title) > 4):
    title = title[4]
    title = title.encode("ASCII")
    title = urllib.unquote(title).decode("utf8")
    title = title.replace("_", " ")
  else:
    title = title[3]

  return title

def url2lang(url):
  lang = url.split("/", 3)[2]
  lang = lang.split(".")[0]
  
  return lang

class WikipediaPage:
  def __init__(self, title=None):
    self.ready = False
    self.query = None
    self.page = None
    self.problem = None

    self.lang = "en"

    if (title):
      title = title.strip()

      r = self.fetch_from_title(title)

      if (r["problem"] != None):
        self.problem = r["problem"]
      else:
        self.page = r["page"]
        self.ready = True


  def fetch_from_title(self, title):
    response = { "page": None, "problem": None, }

    try:
      response["page"] = wikipedia.page(title)

      print "wikipedia page: %s" % response["page"].title.encode("utf8")
      print "\r"

    except wikipedia.exceptions.DisambiguationError:
      response["problem"] = "ambiguity"
      print Fore.YELLOW + "ambiguity"
    except wikipedia.exceptions.PageError:
      response["problem"] = "no match"
      print Fore.YELLOW + "no match"

    return response

  def fetch_from_api_title(self, title, opt_params={ "prop": "info", "inprop": "url" }, lang="en"):
    url = "http://%s.wikipedia.org/w/api.php" % (lang)

    params = {
      "format": "json",
      "action": "query",
      "titles": unicode(title)
      # "rvprop": "content",
      # "redirects": ""
    }

    params = dict(params.items() + opt_params.items())

    r = requests.get(url, params=params)
    # print r.json()

    pages = r.json()["query"]["pages"]

    self.page_id = pages.keys()[0]
    self.title = pages[ self.page_id ]["title"]
    self.lang = lang
    self.url = pages[ self.page_id ]["fullurl"]

    # print r.url
    # print r.text

    return r.json()

  def get_all_editors(self):
    url = "http://%s.wikipedia.org/w/api.php" % (self.lang)

    params = {
      "format": "json",
      "action": "query",
      "titles": self.title,
      "prop": "revisions",
      "rvprop": "user|userid|timestamp|size|ids|sha1",
      "rvlimit": "max",
      "redirects": "",
      "continue": ""
    }

    last = dict()
    revisions = []

    while True:
      current = params.copy()
      current.update(last)

      r = requests.get(url, params=current).json()

      pages = r["query"]["pages"]

      if ("revisions" in pages[ pages.keys()[0] ]):
        # print pages[ pages.keys()[0] ]["revisions"]
        revisions = revisions + pages[ pages.keys()[0] ]["revisions"]

      if 'continue' not in r: break

      last = r["continue"]

    return revisions

  def get_revisions(self, extra_params={}):
    url = "http://%s.wikipedia.org/w/api.php" % (self.lang)

    params = {
      "format": "json",
      "action": "query",
      "titles": self.title,
      "prop": "revisions",
      "rvprop": "user|userid|timestamp|size|ids|sha1|comment|content",
      "rvlimit": "max",
      "redirects": ""
      # ,
      # "continue": ""
    }

    params.update(extra_params)

    # print params

    revisions = []

    while True:
      r = requests.get(url, params=params).json()
      
      # print r
      pages = r["query"]["pages"]
      page = pages[ pages.keys()[0] ]

      revisions += page["revisions"]
      
      if "continue" in r:
        params.update(r["continue"])
      else:
        break

    return revisions

  def get_langlinks(self):
    langlinks = []

    url = "http://%s.wikipedia.org/w/api.php" % (self.lang)

    params = {
      "format": "json",
      "action": "query",
      "titles": self.title,
      "prop": "langlinks",
      "lllimit": 500
    }

    r = requests.get(url, params=params).json()

    # print r

    page = r["query"]["pages"][ r["query"]["pages"].keys()[0] ]

    if ("langlinks" in page):
      langlinks += page["langlinks"]

    return langlinks

  def get_pageviews(self, fr="200712", to=""):
    results = []

    base_url = "http://stats.grok.se/json/%s" % (self.lang)

    if (to == ""):
      year_end = date.today().year
      month_end = date.today().month
    else:
      year_end = int(to[:4])
      month_end = int(to[4:])

    year_start = int(fr[:4])
    month_start = int(fr[4:])

    # print "from: %(year)4d-%(month)02d" % { "year": year_start, "month": month_start }
    # print "to: %(year)4d-%(month)02d" % { "year": year_end, "month": month_end }

    for y in range(year_start, year_end+1):
      m_start = month_start if (y == year_start) else 1
      m_end = month_end if (y == year_end) else 12

      for m in range(m_start, m_end+1):
        month_url = "%(url)s/%(year)4d%(month)02d/%(title)s" % { "url": base_url, "year": y, "month": m, "title": self.title }
        # print month_url
        r = requests.get(month_url).json()
        results.append(r["daily_views"])

    return results


  def links(self):
    links = []

    if (self.ready == True):
      links = self.page.links

    return links


import requests

api_endpoint  = "http://api.wekeypedia.net"

class WeKeyApi:
  def __init__(self):
    self.endpoint = api_endpoint

  def get(self, query):
    return requests.get("%s/%s" % (self.endpoint, query))

  def get_pages(self):
    r = self.get("pages")

    return r.json()


import os

import json

class Dataset:
  def __init__(self, dir=""):
    self.dir = dir

  def get_revision_content(self, page, rev_id):
    f = open( "%s/%s/revisions/%s.json" % (self.dir,page,rev_id), "r")

    return json.load(f)[0]["*"]


from distutils.core import setup
setup(
  name = 'wekeypedia',
  packages = ['wekeypedia'], # this must be the same as the name above
  version = '0.1.2',
  description = 'toolkit to build datasets around wikipedia pages and to compute extra metrics',
  author = 'tam kien duong',
  author_email = 'tk@deveha..com',
  url = 'https://github.com/wekeypedia/toolkit-python', # use the URL to the github repo
  download_url = 'https://github.com/WeKeyPedia/toolkit-python/archive/0.1.2.tar.gz', # I'll explain this in a second
  keywords = ['wikipedia', 'information retrieval', 'api', "data"], # arbitrary keywords
  classifiers = [],
)


from pymongo import MongoClient
from pymongo.errors import DuplicateKeyError

import json

class Dataset():
  def __init__(self, mongo_conf):
    client = MongoClient("mongodb://%s/" % (mongo_conf))

    self.db = client["datasets"]

  def write(self, key, value):
    # print key
    # print json.JSONEncoder().encode(value)

    item = {
      "url": key,
      "dataset": value
    }

    try:
      print "inserting: %s" % (key)
      self.db.datasets.insert(item)
    except DuplicateKeyError:
      # print "duplicated entry: %s" % (key)
      pass

  def delete(self,key):
    self.db.datasets.remove({ "url": key })

  def read(self, key):
    item = {
      "url": key
    }

    value = self.db.datasets.find_one(item)

    return value

  def find(self, query, filter={}):
    return self.db.datasets.find(query, filter)


# -*- coding: utf-8 -*-

import requests

class WikipediaUser:
  def __init__(self, lang="en", name=None):
    self.lang = lang
    self.name = name

  def fetch_contribs(self):
    url = "http://%s.wikipedia.org/w/api.php" % (self.lang)

    contribs = []

    params = {
      "action":"query",
      "format": "json",
      "list":"usercontribs",
      "ucuser": self.name,
      "uclimit": "500",
      "continue": ""
    }

    while True:
      r = requests.get(url, params=params).json()
      contribs += r["query"]["usercontribs"]

      if "continue" in r:
#        print r["continue"]
        params.update(r["continue"])
      else:
        break

    return contribs


import mwparserfromhell as mw

class Mediawiki:
  def __init__(self, txt=""):
    self.text = self.parse(txt)

  def parse(self,txt):
    return mw.parse(txt)


  def get_headings(self):
    return self.text.filter_headings()

  def get_blocks(self):
    sections = []
    structure = []

    nodes = self.text.get_sections(include_lead=True, flat=True)

    n_index = 0

    for n in nodes:
      section = []

      node = mw.parse(n)

      # print ""

      start_at = 1
      previous_level = 2

      if len(node.filter_headings())>0:
        h = node.filter_headings()[0]

        # print h.level
        # print h.title

        # if(h.level <= previous_level):
        #   structure.append([ h.level ])
        # else:
        #   structure[-1].append([ h.level ])
        
        previous_level = h.level

        section.append( len( h.title ) )
      else:
        section.append(0)
        start_at = 0

      stripped_text = node.strip_code()

      # structure.append( "%s:%s" % (previous_level, n_index) )
      structure.append( previous_level )

      paragraphs = stripped_text.split("\n\n")[start_at:]

      for p in paragraphs:
        for w in p.split(" "):
          section.append(len(w))

        section.append(0)

      # print section
      sections.append(section)

      n_index += 1

    return sections, structure